# Copyright (C) 2019 Majormode.  All rights reserved.
#
# This software is the confidential and proprietary information of
# Majormode or one of its subsidiaries.  You shall not disclose this
# confidential information and shall use it only in accordance with the
# terms of the license agreement or other applicable agreement you
# entered into with Majormode.
#
# MAJORMODE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
# OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
# TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE, OR NON-INFRINGEMENT.  MAJORMODE SHALL NOT BE LIABLE FOR ANY
# LOSSES OR DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
# OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.

from majormode.perseus.constant.obj import ObjectStatus
from majormode.perseus.service.base_rdbms_service import BaseRdbmsService
from majormode.perseus.utils import cast
from majormode.xeberus.constant.sim import BillingFrequency
from majormode.xeberus.constant.sim import BillingMethod
from majormode.xeberus.constant.sim import Ownership
from majormode.xeberus.constant.sim import PaymentMethod


class SimCardService(BaseRdbmsService):
    def get_sim_card(
            self,
            iccid,
            check_status=False,
            connection=None,
            include_billing_info=False,
            include_data_info=False):
        """
        Return the specified SIM card worth of extended information.


        :param iccid: Integrated Circuit Card IDentifier (ICCID) stored in the
            SIM card.

        :param check_status: indicate whether to check the current status of
            this SIM card and raise an exception if this SIM card is not
            enabled.

        :param connection: A `RdbmsConnection` object to be used supporting
            the Python clause `with ...:`.

        :param include_billing_info: Indicate whether to include billing
            information about this SIM card.
            
        :param include_data_info: Indicate whether to include data plan
            information about this SIM card.


        :return: an object containing the following attributes:

            * `balance` (optional): Current balance amount of a prepaid SIM card.
              This information is defined when the card owner is the service
              provider and when using a prepaid billing method.  This attribute is
              provided only if the argument `include_billing_info` is `true`.

            * `billing_frequency` (optional): An item of the enumeration
              `BillingFrequency` that indicates the billing frequency for the data
              service.  This information is defined when the card owner is the
              service provider.  This attribute is provided only if the argument
              `include_billing_info` is `true`.

            * `billing_method` (optional): An item of the enumeration
              `BillingMethod` that indicates the billing method of this SIM card.
              This attribute is provided only if the argument
              `include_billing_info` is `true`.

            * `billing_time` (optional): Time of the last billing for the data
              service.  Monthly subscription charges generally commence at this
              time. This attribute is provided only if the argument
              `include_billing_info` is `true`.

            * `ownership` (required): An item of the enumeration `Ownership`
              that indicates who the owner of this SIM card is, and therefore who is
              responsible for paying the SIM card fee to the carrier

            * `creation_time` (required): Time when this SIM card has been
              registered to the platform.

            * `currency_code` (optional): ISO 4217 alphabetical code representing
              the currency of the SIM card's balance and data fee.  This alphabetic
              code is based on another ISO standard, ISO 3166, which lists the codes
              for country names.  The first two letters of the ISO 4217 three-letter
              code are the same as the code for the country name, and where possible
              the third letter corresponds to the first letter of the currency name.
              This attribute is provided only if at least one of the arguments
              `include_billing_info` and `include_data_info` is `true`.

            * `data_fee` (optional): Fee for the data plan depending on the
              billing frequency.  This information is defined when the card owner is
              the service provider and when using a prepaid billing method.  This
              attribute is provided only if the argument `include_data_info` is
              `true`.

            * `data_limit` (optional): Monthly data usage in MB the plan allows,
              or `None` if unlimited.  Once data usage exceeds this limit, the
              customer may be charged a data overage rate.  This attribute is
              provided only if the argument `include_data_info` is `true`.

            * `data_used` (optional): Current amount of data in MB that has been
              used since the last top-up.  This attribute is provided only if the
              argument `include_data_info` is `true`.

            * `iccid` (required): Integrated Circuit Card IDentifier (ICCID)
              stored in the SIM card and also engraved or printed on the SIM card
              body during a process called personalization.  The ICCID is defined by
              the ITU-T recommendation E.118 as the Primary Account Number.  It is
              also known as the serial number of the Subscriber Identity Module
              (SIM) card of the mobile phone, or SIMID.

            * `imsi` (required): International Mobile Subscriber Identity (IMSI)
              as stored in the SIM card.  The first 3 digits are the mobile country
              code (MCC), which are followed by the mobile network code (MNC),
              either 2 digits (European standard) or 3 digits (North American
              standard).

            * `msisdn` (required): Mobile Subscriber Integrated Services Digital
              Network Number (MSISDN) currently associated to the SIM card.

            * `object_status` (required): An item of the enumeration
              `ObjectStatus` that indicates the current status of this SIM card.

            * `operator_code` (required): Identification of the mobile network
              operator (carrier) that provides the SIM card.  This identification is
              composed of the mobile country code (MCC) used in combination with the
              mobile network code (MNC) of this carrier.

              The mobile country code consists of 3 decimal digits and the mobile
              network code consists of 2 or 3 decimal digits.  The first digit of
              the mobile country code identifies the geographic region as follows
              (the digits 1 and 8 are not used):

              * `0`: Test networks

              * `2`: Europe

              * `3`: North America and the Caribbean

              * `4`: Asia and the Middle East

              * `5`: Oceania

              * `6`: Africa

              * `7`: South and Central America

              * `9`: Worldwide (Satellite, Air - aboard aircraft, Maritime -
                aboard ships, Antarctica)

            * `payment_method (optional): An item of the enumeration
              `PaymentMethod` that indicates the payment method to be used to
              increase the remaining balance of the SIM card.  This information is
               defined when the card owner is the service provider and when using a
               prepaid billing method.  This attribute is provided only if the
               argument `include_billing_info` is `true`.

            * `unlimited_data` (optional): Indicate whether the data plan includes
              unlimited data.  The carrier may reduce the bandwidth when data usage
              exceeds the data speed reduction threshold (cf. `data_limit`).
              This attribute is provided only if the argument `include_data_info`
              is `true`.

            * `update_time` (required): Time of the most recent modification of
              any variable attribute of this SIM card, such as its status, the most
              recent top-up of the SIM balance.


        :raise DeletedObjectException: if the SIM card cannot be used anymore
            (expired, destroyed, lost, etc.).

        :raise DisabledObjectException: if the SIM card is not currently used;
            its properties, such as the billing and data plan information, may
            not be up-to-date.

        :raise UndefinedObjectException: if no SIM card has been registered to
            the platform with the specified ICCID.
        """
        with self.acquire_rdbms_connection(auto_commit=False, connection=connection) as connection:
            cursor = connection.execute(
                """
                SELECT 
                    iccid,
                    imsi,
                    operator_code,
                    msisdn,
                    ownership,
                    billing_method,
                    billing_frequency,
                    billing_time,
                    payment_method,
                    data_limit,
                    unlimited_data,
                    data_used,
                    data_fee,
                    balance,
                    currency_code,
                    object_status,
                    creation_time,
                    update_time
                  FROM 
                    sim_card
                  WHERE 
                    iccid = %(iccid)s
                """,
                {
                    'iccid': iccid
                })

            row = cursor.fetch_one()
            if row is None:
                raise self.UndefinedObjectException(f"undefined SIM card with ICCID {iccid}")

            sim_card = row.get_object({
                'activation_time': cast.string_to_timestamp,
                'creation_time': cast.string_to_timestamp,
                'billing_frequency': BillingFrequency,
                'billing_method': BillingMethod,
                'billing_time': cast.string_to_timestamp,
                'ownership': Ownership,
                'object_status': ObjectStatus,
                'payment_method': PaymentMethod,
                'update_time': cast.string_to_timestamp
            })

            if check_status:
                if sim_card.object_status == ObjectStatus.deleted:
                    raise self.DeletedObjectException(f"the SIM card with ICCID {iccid} has been banned")
                elif sim_card.object_status == ObjectStatus.disabled:
                    raise self.DisabledObjectException(f"the SIM card with ICCID {iccid} has been disabled")

            if not include_billing_info:
                del sim_card.billing_frequency, \
                    sim_card.billing_method, \
                    sim_card.payment_method, \
                    sim_card.balance

            if not include_data_info:
                del sim_card.data_limit, \
                    sim_card.unlimited_data, \
                    sim_card.data_used, \
                    sim_card.data_fee

            if not (include_billing_info or include_data_info):
                del sim_card.currency_code

            return sim_card

    def register_sim_card(self, app_id, iccid, imsi, connection=None):
        """
        Register a new SIM card.  An administrator of the platform will have
        to complete its properties, such as the billing and data plan
        information, in order to active this SIM card.


        :param app_id: identification of the client application such as a Web,
            a desktop, or a mobile application, that accesses the service.

        :param iccid: Integrated Circuit Card IDentifier (ICCID) stored in the
            SIM card and also engraved or printed on the SIM card body during
            a process called personalization.  The ICCID is defined by the
            ITU-T recommendation E.118 as the Primary Account Number.  It is
            also known as the serial number of the Subscriber Identity Module
            (SIM) card of the mobile phone, or SIMID.

        :param imsi: International Mobile Subscriber Identity (IMSI) as stored
            in the SIM card.

        :param connection: A `RdbmsConnection` object to be used supporting
            the Python clause `with ...:`.


        :return: An object returned by the function `get_sim_card`.
        """
        with self.acquire_rdbms_connection(auto_commit=True, connection=connection) as connection:
            connection.execute(
                """
                INSERT INTO sim_card(
                    iccid,
                    imsi,
                    operator_code,
                    object_status)
                  VALUES 
                    (%(iccid)s,
                     %(imsi)s,
                     %(operator_code)s,
                     %(OBJECT_STATUS_PENDING)s)
                """,
                {
                    'OBJECT_STATUS_PENDING': ObjectStatus.pending,
                    'iccid': iccid,
                    'imsi': imsi,
                    'operator_code': imsi[:5]
                })

        return self.get_sim_card(iccid, connection=connection)

    def set_sim_card(
            self,
            app_id,
            account_id,
            iccid,
            balance=None,
            billing_frequency=None,
            billing_method=None,
            billing_time=None,
            connection=None,
            ownership=None,
            currency_code=None,
            data_fee=None,
            data_limit=None,
            data_used=None,
            payment_method=None,
            unlimited_data=None):
        """
        Specify the properties of the SIM card, such as its billing and data
        plan information.


        :param app_id: identification of the client application such as a Web,
            a desktop, or a mobile application, that accesses the service.

        :param account_id: identification of the account of the user who is
            defining the properties of this SIM card.

        :param iccid: integrated Circuit Card IDentifier (ICCID) stored in the
            SIM card.

        :param balance: current balance amount of a prepaid SIM card.  This
            parameter is required when the card owner is the service provider
            and when using a prepaid billing method.

        :param billing_frequency: an item of the enumeration
            `BillingFrequency` that indicates the billing frequency for the
            data service.  This parameter is required when data are not
            unlimited.

        :param billing_method: an item of the enumeration `BillingMethod`
            that indicates the billing method of this SIM card.  This
            parameter is required when the card owner is the service provider.

        :param billing_time: the time of the last billing for the data
            service.  Monthly subscription charges generally commence at this
            time.  This parameter is required when data are not unlimited.

        :param connection: A `RdbmsConnection` object to be used supporting
            the Python clause `with ...:`.

        :param ownership: an item of the enumeration `Ownership` that
            indicates who the owner of this SIM card is, and therefore who is
            responsible for paying the SIM card fee to the carrier.  Only an
            administrator of the platform can specified this information,
            otherwise the function raises the exception
            `IllegalAccessException`.

        :param currency_code: ISO 4217 alphabetical code representing the
            currency of the SIM card's balance and data fee.  This alphabetic
            code is based on another ISO standard, ISO 3166, which lists the
            codes for country names.  The first two letters of the ISO 4217
            three-letter code are the same as the code for the country name,
            and where possible the third letter corresponds to the first
            letter of the currency name.  This parameter is required when the
            parameter `data_fee` or `balance` is passed to this function.

        :param data_fee: Fee for the data plan depending on the billing
            frequency.  This parameter is only when the card owner is the
            service provider and when the data are not unlimited.

        :param data_limit: monthly data usage in MB the plan allows.  Once
            data usage exceeds this limit, the customer may be charged a data
            overage rate.  This parameter is required when data are not
            unlimited.

        :param data_used: current amount of data in MB that has been used
            since the last billing time.  This parameter is required if data
            are not unlimited.

        :param payment_method: an item of the enumeration `PaymentMethod`
            that indicates the payment method to be used to increase the
            remaining balance of the SIM card.  This parameter is required
            when the card owner is the service provider and when using a
            prepaid billing method.

        :param unlimited_data: indicate whether the data plan includes
            unlimited data.  The carrier may reduce the bandwidth when data
            usage exceeds the data speed reduction threshold (cf. parameter
            `data_limit`).  This parameter is required when the user on
            behalf of whom this function is an administrator of the platform
            and the parameter `ownership` identifies the service provider,
            or when this user is the customer.


        :raise DeletedObjectException: if the SIM card cannot be used anymore
            (expired, destroyed, lost, etc.).

        :raise DisabledObjectException: if the SIM card is not currently used;
            its properties, such as the billing and data plan information, may
            not be up-to-date.

        :raise IllegalAccessException: if the argument `ownership` has been
            passed to this function while the user is not an administrator of
            the platform.

        :raise InvalidArgumentException: if some arguments have not been
            passed to this function while they are required.

        :raise InvalidOperationException: if the argument `ownership` has
            been passed to this function while the SIM card is active.  It's
            not allowed to change the ownership of a SIM card in that context.

        :raise UndefinedObjectException: if no SIM card has been registered to
            the platform with the specified ICCID.
        """
        # Retrieve the current properties of the SIM card.
        sim_card = self.get_sim_card(iccid, check_status=True, connection=connection)

        # Check that the ownership has been defined or is being defined, which
        # is required to define the other properties of the SIM card.
        if not sim_card.ownership and not ownership:
            raise self.InvalidArgumentException('Card ownership is required')

        # # The owner of the SIM card can be specified only by an administrator
        # # of the platform.
        # from majormode.xeberus.service.xeberus.xeberus_service import \
        #     XeberusService
        #
        # is_administrator = TeamService().is_administrator(account_id, XeberusService.XEBERUS_TEAM.team_id)
        # if ownership:
        #     TeamService()._assert_administrator(account_id, XeberusService.XEBERUS_TEAM.team_id)
        #
        #     if sim_card.object_status == ObjectStatus.enabled:
        #         raise self.InvalidOperationException('It is not possible to change the ownership of an active SIM card')

        # Check that the required parameters have been properly defined.
        if billing_method is None and ownership == Ownership.service:
            raise self.InvalidArgumentException('billing method is required')

        # if unlimited_data is None and (ownership == Ownership.customer or \
        #     (is_administrator and ownership == Ownership.service)):
        #     raise self.InvalidArgumentException('The indicator whether data are unlimited is required')

        if billing_frequency is None and unlimited_data is None:
            raise self.InvalidArgumentException('billing frequency is required')

        if billing_time is None and unlimited_data is None:
            raise self.InvalidArgumentException('last billing time is required')

        if payment_method is None and ownership == Ownership.service and billing_method == BillingMethod.prepaid:
            raise self.InvalidArgumentException('billing method is required')

        if data_limit is None and not unlimited_data:
            raise self.InvalidArgumentException('data limit is required')

        if data_used is None and not unlimited_data:
            raise self.InvalidArgumentException('data used is required')

        if data_fee is None and ownership == Ownership.service:
            raise self.InvalidArgumentException('data fee is required')

        if balance is None and ownership == Ownership.service and billing_method == BillingMethod.prepaid:
            raise self.InvalidArgumentException('balance is required')

        if currency_code is None and (data_fee or balance):
            raise self.InvalidArgumentException('currency code is required')

        # Save the properties of the SIM card that have been specified, i.e.,
        # not null.
        fields = {
            'balance': balance,
            'billing_frequency': billing_frequency,
            'billing_method': billing_method,
            'billing_time': billing_time,
            'ownership': ownership,
            'currency_code': currency_code,
            'data_fee': data_fee,
            'data_limit': data_limit,
            'data_used': data_used,
            'payment_method': payment_method,
            'unlimited_data': unlimited_data
        }

        defined_fields = dict([
            (name, value)
            for (name, value) in fields.iteritems()
            if value is not None
        ])

        if defined_fields:
            update_set_expression = ','.join([
                '%s = %%(%s)s' % (name, name)
                for name in defined_fields.iterkeys()
            ])

            defined_fields['iccid'] = iccid

            with self.acquire_rdbms_connection(True) as connection:
                connection.execute(
                    """
                    UPDATE 
                        sim_card
                      SET 
                        update_time = current_timestamp,""" + update_set_expression + """
                      WHERE 
                        iccid = %(iccid)s
                    """,
                    defined_fields)



# Copyright (C) 2019 Majormode.  All rights reserved.
#
# This software is the confidential and proprietary information of
# Majormode or one of its subsidiaries.  You shall not disclose this
# confidential information and shall use it only in accordance with the
# terms of the license agreement or other applicable agreement you
# entered into with Majormode.
#
# MAJORMODE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
# OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
# TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE, OR NON-INFRINGEMENT.  MAJORMODE SHALL NOT BE LIABLE FOR ANY
# LOSSES OR DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
# OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.

from majormode.perseus.constant.http import HttpMethod
from majormode.perseus.model.geolocation import GeoPoint
from majormode.perseus.service.base_http_handler import HttpRequest
from majormode.perseus.service.base_http_handler import HttpRequestHandler
from majormode.perseus.service.base_http_handler import http_request
from majormode.xeberus.model.tracker import BatteryStateChangeEvent
from majormode.xeberus.model.tracker import LocationUpdate

from majormode.xeberus.service.tracker.tracker_service import TrackerService


class TrackerServiceHttpRequestHandler(HttpRequestHandler):
    @http_request(r'^/tracker/(device_id)/activation/(activation_code)$',
                  http_method=HttpMethod.POST,
                  authentication_required=False,
                  signature_required=True)
    def activate_device(self, request, device_id, activation_code):
        # Check whether the information of the agent application has been
        # correctly passed.
        if not request.client_application:
            raise TrackerService.InvalidOperationException(
                "the information of the tracker agent application has not be given (cf. User-Agent HTTP header)")

        imei = request.get_argument(
            'imei',
            data_type=HttpRequest.ArgumentDataType.string,
            is_required=True)

        mac_address = request.get_argument(
            'mac_address',
            data_type=HttpRequest.ArgumentDataType.macaddr,
            is_required=True)

        battery_level = request.get_argument(
            'battery_level',
            data_type=HttpRequest.ArgumentDataType.decimal,
            is_required=True)

        is_battery_plugged = request.get_argument(
            'is_battery_plugged',
            data_type=HttpRequest.ArgumentDataType.boolean,
            is_required=True)

        location = request.get_argument(
            'location',
            data_type=HttpRequest.ArgumentDataType.object,
            object_class=GeoPoint,
            is_required=False)

        return TrackerService().activate_device(
            request.app_id,
            device_id,
            imei,
            activation_code,
            mac_address,
            request.client_application,
            battery_level,
            is_battery_plugged,
            location)

    @http_request(r'^/tracker/(device_id)/battery/event$',
                  http_method=HttpMethod.POST,
                  authentication_required=False,
                  signature_required=True)
    def report_battery_events(self, request, device_id):
        if not isinstance(request.body, list):
            raise TrackerService.InvalidArgumentException("The message body MUST be a list of battery events")

        seed = request.get_argument(
            's',
            data_type=HttpRequest.ArgumentDataType.string,
            argument_passing=HttpRequest.ArgumentPassing.query_string,
            is_required=False)

        events = [
            BatteryStateChangeEvent.from_json(payload)
            for payload in request.body
        ]

        return TrackerService().report_battery_events(
            request.app_id,
            device_id,
            events,
            account_id=request.session and request.session.account_id,
            seed=seed)

    @http_request(r'^/tracker/(device_id)/keepalive$',
                  http_method=HttpMethod.POST,
                  authentication_required=False,
                  signature_required=True)
    def report_keepalive(self, request, device_id):
        seed = request.get_argument(
            's',
            data_type=HttpRequest.ArgumentDataType.string,
            argument_passing=HttpRequest.ArgumentPassing.query_string,
            is_required=True)

        battery_level = request.get_argument(
            'battery_level',
            data_type=HttpRequest.ArgumentDataType.decimal,
            is_required=False)

        is_battery_plugged = request.get_argument(
            'is_battery_plugged',
            data_type=HttpRequest.ArgumentDataType.boolean,
            is_required=False)

        location = request.get_argument(
            'location',
            data_type=HttpRequest.ArgumentDataType.object,
            object_class=GeoPoint,
            is_required=False)

        network_type = request.get_argument(
            'network_type',
            data_type=HttpRequest.ArgumentDataType.string,
            is_required=False)

        mobile_exchanged_byte_count = request.get_argument(
            'ex_bytes',
            data_type=HttpRequest.ArgumentDataType.integer,
            is_required=False)

        return TrackerService().report_keepalive(
            request.app_id,
            device_id,
            seed,
            account_id=request.session and request.session.account_id,
            battery_level=battery_level,
            is_battery_plugged=is_battery_plugged,
            location=location,
            mobile_exchanged_byte_count=mobile_exchanged_byte_count,
            network_type=network_type)

    @http_request(r'^/tracker/(device_id)/location$',
                  http_method=HttpMethod.POST,
                  authentication_required=False,
                  signature_required=True)
    def report_location_updates(self, request, device_id):
        if not isinstance(request.body, list):
            raise TrackerService.InvalidArgumentException("The message body MUST be a list of location updates")

        seed = request.get_argument(
            's',
            data_type=HttpRequest.ArgumentDataType.string,
            argument_passing=HttpRequest.ArgumentPassing.query_string,
            is_required=False)

        team_id = request.get_argument(
            'team_id',
            data_type=HttpRequest.ArgumentDataType.string,
            argument_passing=HttpRequest.ArgumentPassing.query_string,
            is_required=False)

        events = [
            LocationUpdate.from_json(payload, device_id)
            for payload in request.body
        ]

        return TrackerService().report_location_updates(
            request.app_id,
            device_id,
            events,
            account_id=request.session and request.session.account_id,
            seed=seed,
            team_id=team_id)

    @http_request(r'^/tracker/activation/request$',
                  http_method=HttpMethod.POST,
                  authentication_required=True,
                  signature_required=True)
    def request_activation_code(self, request):
        activation_code_duration = request.get_argument(
            'duration',
            data_type=HttpRequest.ArgumentDataType.integer,
            is_required=False)

        team_id = request.get_argument(
            'team_id',
            data_type=HttpRequest.ArgumentDataType.uuid,
            is_required=True)

        return TrackerService().request_activation_code(
            request.app_id,
            request.account_id,
            team_id,
            activation_code_duration=activation_code_duration)

    @http_request(r'^/tracker/(device_id)/handshake$',
                  http_method=HttpMethod.POST,
                  authentication_required=False,
                  signature_required=True)
    def shake_hands(self, request, device_id):
        # @TODO: REMOVE THIS PATCH ONCE THE BUG FOUND AND FIXED!
        user_agent = request.get_header('User-Agent', is_required=False)
        with open('/home/devops/user_agents.txt', 'at') as fd:
            fd.write(user_agent + '\n')
        if request.client_application is None:
            from majormode.perseus.model.app import ClientApplication
            from majormode.perseus.model.version import Version
            request.client_application = ClientApplication(
                user_agent,
                Version(1, 0, 0),
                'Android',
                Version(1, 0, 0),
                'FooBar')

        # Check whether the information of the agent application has been
        # correctly passed.
        if not request.client_application:
            raise TrackerService.InvalidOperationException(
                "the information of the tracker agent application has not be given (cf. User-Agent HTTP header)")

        battery_level = request.get_argument(
            'battery_level',
            data_type=HttpRequest.ArgumentDataType.decimal,
            is_required=True)

        is_battery_plugged = request.get_argument(
            'is_battery_plugged',
            data_type=HttpRequest.ArgumentDataType.boolean,
            is_required=True)

        is_battery_present = request.get_argument(
            'is_battery_present',
            data_type=HttpRequest.ArgumentDataType.boolean,
            is_required=True)

        location = request.get_argument(
            'location',
            data_type=HttpRequest.ArgumentDataType.object,
            object_class=GeoPoint,
            is_required=False)

        mac_address = request.get_argument(
            'mac_address',
            data_type=HttpRequest.ArgumentDataType.string,
            is_required=True)

        print(f'battery_level: {battery_level}')
        print(f'is_battery_plugged: {is_battery_plugged}')
        print(f'is_battery_present: {is_battery_present}')
        print(f'mac_address: {mac_address}')

        return TrackerService().shake_hands(
            request.app_id,
            device_id,
            mac_address,
            request.client_application,
            is_battery_present,
            is_battery_plugged,
            battery_level,
            location=location)

































    # @http_request(r'^/device/(device_id)/alert_message$',
    #               http_method=HttpRequest.HttpMethod.POST,
    #               authentication_required=False,
    #               signature_required=True)
    # def on_device_alert_message(self, request, device_id):
    #     seed = request.get_argument('s',
    #         data_type=HttpRequest.ArgumentDataType.string,
    #         argument_passing=HttpRequest.ArgumentPassing.query_string,
    #         is_required=True)
    #
    #     if not isinstance(request.body, list):
    #         raise XeberusService.InvalidArgumentException()
    #
    #     state_changes = [ AlertStateChangeEvent.from_json(payload)
    #             for payload in request.body ]
    #
    #     return XeberusService().on_device_alert_message(request.app_id, device_id, seed, state_changes)
    #
    #
    #
    #


    #
    #
    #
    #
    #
    #
    #
    #
    #
    #
    #
    #
    #
    # @http_request(r'/device/(device_id)/allocation$',
    #               http_method=HttpRequest.HttpMethod.POST,
    #               authentication_required=True,
    #               signature_required=True)
    # def acquire_device(self, request, device_id):
    #     return XeberusService().acquire_device(request.app_id, request.account_id, device_id)
    #
    #
    # @http_request(r'^/device$',
    #               http_method=HttpRequest.HttpMethod.GET,
    #               authentication_required=True,
    #               signature_required=True)
    # def get_devices(self, request):
    #     """
    #     @deprecated: to be replaced with get_devices_ex
    #     """
    #     limit = request.get_argument('limit',
    #             data_type=HttpRequest.ArgumentDataType.integer,
    #             is_required=False,
    #             default_value=XeberusService.DEFAULT_LIMIT)
    #
    #     offset = request.get_argument('offset',
    #             data_type=HttpRequest.ArgumentDataType.integer,
    #             is_required=False,
    #             default_value=0)
    #
    #     sync_time = request.get_argument('sync_time',
    #             data_type=HttpRequest.ArgumentDataType.timestamp,
    #             is_required=False)
    #
    #     return XeberusService().get_devices(request.app_id, request.account_id,
    #             limit=limit,
    #             offset=offset,
    #             sync_time=sync_time)
    #
    #
    # @http_request(r'^/device_ex$',
    #               http_method=HttpRequest.HttpMethod.GET,
    #               authentication_required=True,
    #               signature_required=True)
    # def get_devices_ex(self, request):
    #     limit = request.get_argument('limit',
    #             data_type=HttpRequest.ArgumentDataType.integer,
    #             is_required=False,
    #             default_value=XeberusService.DEFAULT_LIMIT)
    #
    #     offset = request.get_argument('offset',
    #             data_type=HttpRequest.ArgumentDataType.integer,
    #             is_required=False,
    #             default_value=0)
    #
    #     sync_time = request.get_argument('sync_time',
    #             data_type=HttpRequest.ArgumentDataType.timestamp,
    #             is_required=False)
    #
    #     return XeberusService().get_devices_ex(request.app_id, request.account_id,
    #             limit=limit,
    #             offset=offset,
    #             sync_time=sync_time)
    #
    #
    #
    # # @http_request(r'^/venue/(venue_id:uuid)/confirmation$',
    # #               http_method=HttpRequest.HttpMethod.POST,
    # #               authentication_required=True,
    # #               signature_required=True)
    # # def confirm_suggested_venue(self, request, venue_id):
    # #     return XeberusService().confirm_suggested_venue(request.app_id, request.account_id, venue_id)
    #
    #
    # # @http_request(r'^/venue/(venue_id:uuid)',
    # #               http_method=HttpRequest.HttpMethod.DELETE,
    # #               authentication_required=True,
    # #               signature_required=True)
    # # def delete_venue(self, request, venue_id):
    # #     return XeberusService().delete_venue(request.app_id, request.account_id, venue_id)
    #
    #
    # # @http_request(r'/route')
    # # def add_route(self, request, ):
    # #     name = request.get_argument('name',
    # #             )
    # #
    # #     path = request.get_argument('path',
    # #             data_type=HttpRequest.ArgumentDataType.list,
    # #             item_data_type=HttpRequest.ArgumentDataType.object,
    # #             object_class=GeoPoint,
    # #             is_required=True)
    # #
    # #     waypoints = request.get_argument('waypoints',
    # #             data_type=HttpRequest.ArgumentDataType.list,
    # #             item_data_type=HttpRequest.ArgumentDataType.object,
    # #             object_class=GeoPoint,
    # #             is_required=True)
    # #
    # #     return XeberusService().add_route(request.app_id, request.account_id, path,
    # #             waypoints=waypoints)
    #
    #
    # @http_request(r'^/device/(device_id)/notification$',
    #               http_method=HttpRequest.HttpMethod.GET,
    #               authentication_required=False,
    #               signature_required=True)
    # def get_device_notifications(self, request, device_id):
    #     seed = request.get_argument('s',
    #             data_type=HttpRequest.ArgumentDataType.string,
    #             is_required=True)
    #
    #     start_time = request.get_argument('start_time',
    #             data_type=HttpRequest.ArgumentDataType.timestamp,
    #             is_required=False)
    #     end_time = request.get_argument('end_time',
    #             data_type=HttpRequest.ArgumentDataType.timestamp,
    #             is_required=False)
    #     offset = request.get_argument('offset',
    #             data_type=HttpRequest.ArgumentDataType.integer,
    #             is_required=False,
    #             default_value=0)
    #     limit = request.get_argument('limit',
    #             data_type=HttpRequest.ArgumentDataType.integer,
    #             is_required=False,
    #             default_value=XeberusService.DEFAULT_LIMIT)
    #     include_read = request.get_argument('include_read',
    #             data_type=HttpRequest.ArgumentDataType.boolean,
    #             is_required=False,
    #             default_value=False)
    #     mark_read = request.get_argument('mark_read',
    #             data_type=HttpRequest.ArgumentDataType.boolean,
    #             is_required=False,
    #             default_value=True)
    #
    #     return XeberusService().get_device_notifications(request.app_id, device_id, seed,
    #         start_time=start_time, end_time=end_time,
    #         offset=offset, limit=limit,
    #         include_read=include_read, mark_read=mark_read)
    #
    #
    # @http_request(r'^/device/registration$',
    #               http_method=HttpRequest.HttpMethod.GET,
    #               authentication_required=True,
    #               signature_required=True)
    # def get_device_registrations(self, request):
    #     limit = request.get_argument('limit',
    #         data_type=HttpRequest.ArgumentDataType.integer,
    #         is_required=False,
    #         default_value=XeberusService.DEFAULT_LIMIT)
    #     offset = request.get_argument('offset',
    #         data_type=HttpRequest.ArgumentDataType.integer,
    #         is_required=False,
    #         default_value=0)
    #
    #     return XeberusService().get_device_registrations(request.app_id, request.account_id,
    #         limit=limit, offset=offset)
    #
    #
    # @http_request(r'^/device/alert$',
    #               http_method=HttpRequest.HttpMethod.GET,
    #               authentication_required=True,
    #               signature_required=True)
    # def get_devices_alerts(self, request):
    #     device_id = request.get_argument('device_id',
    #         data_type=HttpRequest.ArgumentDataType.list)
    #     limit = request.get_argument('limit',
    #         data_type=HttpRequest.ArgumentDataType.integer,
    #         is_required=False,
    #         default_value=XeFuryService.DEFAULT_LIMIT)
    #     offset = request.get_argument('offset',
    #         data_type=HttpRequest.ArgumentDataType.integer,
    #         is_required=False,
    #         default_value=0)
    #     start_time = request.get_argument('start_time',
    #         data_type=HttpRequest.ArgumentDataType.timestamp,
    #         is_required=False)
    #     return XeberusService().get_devices_alerts(request.app_id, request.account_id, device_id,
    #             start_time=start_time, limit=limit, offset=offset)
    #
    #
    # @http_request(r'^/device/(device_id)/location$',
    #               http_method=HttpRequest.HttpMethod.GET,
    #               authentication_required=True,
    #               signature_required=True)
    # def get_device_locations(self, request, device_id):
    #     start_time = request.get_argument('start_time',
    #             data_type=HttpRequest.ArgumentDataType.timestamp,
    #             is_required=True)
    #
    #     end_time = request.get_argument('end_time',
    #             data_type=HttpRequest.ArgumentDataType.timestamp,
    #             is_required=True)
    #
    #     limit = request.get_argument('limit',
    #             data_type=HttpRequest.ArgumentDataType.integer,
    #             is_required=False,
    #             default_value=XeberusService.DEFAULT_LIMIT)
    #
    #     offset = request.get_argument('offset',
    #             data_type=HttpRequest.ArgumentDataType.integer,
    #             is_required=False,
    #             default_value=0)
    #
    #     return XeberusService().get_device_locations(request.app_id, request.account_id,
    #             device_id, start_time, end_time,
    #             limit=limit,
    #             offset=offset)
    #
    #
    # @http_request(r'^/device/location/recent$',
    #               http_method=HttpRequest.HttpMethod.GET,
    #               authentication_required=True,
    #               signature_required=True)
    # def get_devices_recent_locations(self, request):
    #     devices = dict([ (device_id, cast.string_to_timestamp(timestamp))
    #             for (device_id, timestamp) in [ device.split('@')
    #                 for device in request.get_argument('devices',
    #                         data_type=HttpRequest.ArgumentDataType.list,
    #                         item_data_type=HttpRequest.ArgumentDataType.string,
    #                         is_required=True) ] ])
    #
    #     fix_age = request.get_argument('fix_age',
    #             data_type=HttpRequest.ArgumentDataType.integer,
    #             is_required=False,
    #             default_value=XeberusService.DEFAULT_FIX_AGE)
    #
    #     limit = request.get_argument('limit',
    #             data_type=HttpRequest.ArgumentDataType.integer,
    #             is_required=False,
    #             default_value=XeberusService.DEFAULT_LIMIT)
    #
    #     return XeberusService().get_devices_recent_locations(request.app_id, request.account_id, devices,
    #             fix_age=fix_age,
    #             limit=limit)
    #
    #
    # @http_request(r'^/vehicle/message$',
    #               http_method=HttpRequest.HttpMethod.GET,
    #               authentication_required=False,
    #               signature_required=True)
    # def get_vehicle_models(self, request):
    #     make = request.get_argument('make',
    #         data_type=HttpRequest.ArgumentDataType.string,
    #         is_required=False)
    #     year = request.get_argument('year',
    #         data_type=HttpRequest.ArgumentDataType.integer,
    #         is_required=False)
    #     sync_time = request.get_argument('sync_time',
    #         data_type=HttpRequest.ArgumentDataType.timestamp,
    #         is_required=False)
    #
    #     return XeberusService().get_vehicle_models(request.app_id,
    #             make=make, year=year, sync_time=sync_time)
    #
    #
    # # @http_request(r'^/venue$',
    # #               http_method=HttpRequest.HttpMethod.GET,
    # #               authentication_required=False,
    # #               signature_required=True)
    # # def get_venues(self, request):
    # #     include_address = request.get_argument('include_address',
    # #             data_type=HttpRequest.ArgumentDataType.boolean,
    # #             is_required=False,
    # #             default_value=False)
    # #
    # #     include_contacts = request.get_argument('include_contacts',
    # #             data_type=HttpRequest.ArgumentDataType.boolean,
    # #             is_required=False,
    # #             default_value=False)
    # #
    # #     include_stopovers = request.get_argument('include_stopovers',
    # #             data_type=HttpRequest.ArgumentDataType.boolean,
    # #             is_required=False,
    # #             default_value=False)
    # #
    # #     limit = request.get_argument('limit',
    # #             data_type=HttpRequest.ArgumentDataType.integer,
    # #             is_required=False,
    # #             default_value=XeberusService.DEFAULT_VENUE_LIMIT)
    # #
    # #     locale = request.get_argument('locale',
    # #             data_type=HttpRequest.ArgumentDataType.locale,
    # #             is_required=False,
    # #             default_value=DEFAULT_LOCALE)
    # #
    # #     statuses = request.get_argument('statuses',
    # #             data_type=HttpRequest.ArgumentDataType.list,
    # #             item_data_type=HttpRequest.ArgumentDataType.integer,
    # #             is_required=False)
    # #
    # #     sync_time = request.get_argument('sync_time',
    # #         data_type=HttpRequest.ArgumentDataType.timestamp,
    # #         is_required=False)
    # #
    # #     team_id = request.get_argument('team_id',
    # #         data_type=HttpRequest.ArgumentDataType.uuid,
    # #         is_required=False)
    # #
    # #     return XeberusService().get_venues(request.app_id, request.account_id,
    # #             include_address=include_address,
    # #             include_contacts=include_contacts,
    # #             #include_stopovers=include_stopovers,
    # #             statuses=statuses,
    # #             limit=limit,
    # #             locale=locale,
    # #             sync_time=sync_time,
    # #             team_id=team_id)
    #
    #
    #
    #
    #
    #
    #
    # @http_request(r'^/device/(device_id)/error$',
    #               http_method=HttpRequest.HttpMethod.POST,
    #               authentication_required=False,
    #               signature_required=True)
    # def on_error_raised(self, request, device_id):
    #     if not isinstance(request.body, list):
    #         raise XeberusService.InvalidArgumentException()
    #
    #     seed = request.get_argument('s',
    #         data_type=HttpRequest.ArgumentDataType.string,
    #         argument_passing=HttpRequest.ArgumentPassing.query_string,
    #         is_required=False)
    #
    #     return XeberusService().on_error_raised(request.app_id, device_id, seed,
    #             [ ErrorReport.from_json(payload, device_id) for payload in request.body ])
    #
    #
    #
    # @http_request(r'^/payment/cash$',
    #               http_method=HttpRequest.HttpMethod.POST,
    #               authentication_required=True,
    #               signature_required=True)
    # def register_cash_payment(self, request):
    #     customer_account_id = request.get_argument('account_id',
    #         data_type=HttpRequest.ArgumentDataType.uuid,
    #         is_required=True)
    #     amount = request.get_argument('amount',
    #         data_type=HttpRequest.ArgumentDataType.decimal,
    #         is_required=True)
    #     currency = request.get_argument('currency',
    #         data_type=HttpRequest.ArgumentDataType.string,
    #         is_required=True)
    #
    #     return XeberusService().register_cash_payment(request.app_id, request.account_id,
    #         customer_account_id, amount, currency)
    #
    #
    # @http_request(r'^/prospect/vehicle/registration$',
    #               http_method=HttpRequest.HttpMethod.POST,
    #               authentication_required=False,
    #               signature_required=True)
    # def _register_prospect_vehicle_model(self, request):
    #     email_address = request.get_argument('email_address',
    #         data_type=HttpRequest.ArgumentDataType.string,
    #         is_required=True)
    #     model_id = request.get_argument('model_id',
    #         data_type=HttpRequest.ArgumentDataType.uuid,
    #         is_required=False)
    #
    #     make = None if model_id else request.get_argument('make',
    #         data_type=HttpRequest.ArgumentDataType.string,
    #         is_required=True)
    #     model = None if model_id else request.get_argument('message',
    #         data_type=HttpRequest.ArgumentDataType.string,
    #         is_required=False)
    #
    #     return XeberusService()._register_prospect_vehicle_model(request.app_id, email_address,
    #             model_id=model_id,
    #             make=make,
    #             model=model)
    #
    #
    # @http_request(r'^/topup-card$',
    #               http_method=HttpRequest.HttpMethod.POST,
    #               authentication_required=True,
    #               signature_required=True)
    # def register_topup_cards(self, request):
    #     topup_cards = [
    #             Object(credit_amount=payload['credit_amount'],
    #                    currency_code=payload['currency_code'],
    #                    expiration_time=cast.string_to_timestamp(payload.get('expiration_time')),
    #                    magic_code=payload['magic_code'],
    #                    operator_code=payload['operator_code'],
    #                    serial_number=payload['serial_number'])
    #             for payload in request.body ]
    #
    #     return XeberusService().register_topup_cards(request.app_id, request.account_id,
    #                 topup_cards)
    #
    #
    # @http_request(r'^/device/(device_id)/mileage/(mileage:integer)$',
    #               http_method=HttpRequest.HttpMethod.PUT,
    #               authentication_required=True,
    #               signature_required=True)
    # def set_device_mileage(self, request, device_id, mileage):
    #     return XeberusService().set_device_mileage(request.app_id, request.account_id, device_id, mileage)
    #
    #
    # @http_request(r'^/device/(device_id)/picture$',
    #               http_method=HttpRequest.HttpMethod.PUT,
    #               authentication_required=True,
    #               signature_required=True)
    # def set_device_picture(self, request, device_id):
    #     if len(request.uploaded_files) != 1:
    #         raise XeberusService.InvalidOperationException('One and one only picture file MUST be uploaded')
    #
    #     return XeberusService().set_device_picture(request.app_id, request.account_id, device_id, request.uploaded_files[0])
    #
    #
    # @http_request(r'/sim/(imsi)/plan$',
    #               http_method=HttpRequest.HttpMethod.PUT,
    #               authentication_required=True,
    #               signature_required=True)
    # def set_sim_plan(self, request, imsi):
    #     data_amount = request.get_argument('data_amount',
    #             data_type=HttpRequest.ArgumentDataType.integer,
    #             is_required=True)
    #
    #     activation_time = request.get_argument('activation_time',
    #             data_type=HttpRequest.ArgumentDataType.timestamp,
    #             is_required=True)
    #
    #     return XeberusService().set_sim_plan(request.app_id, request.account_id,
    #             imsi, data_amount, activation_time)
    #
    # @http_request(r'^/device/(device_id)/playback/(slot:integer)$',
    #               http_method=HttpRequest.HttpMethod.POST,
    #               authentication_required=True,
    #               signature_required=True)
    # def start_playback(self, request, device_id, slot):
    #     return XeberusService().start_playback(request.app_id, request.account_id, device_id, slot)
    #
    #
    # @http_request(r'^/device/(device_id)/playback$',
    #               http_method=HttpRequest.HttpMethod.DELETE,
    #               authentication_required=True,
    #               signature_required=True)
    # def stop_playback(self, request, device_id):
    #     return XeberusService().stop_playback(request.app_id, request.account_id, device_id)
    #
    #
    # # @http_request(r'^/device/(device_id)/stopover',
    # #               http_method=HttpRequest.HttpMethod.POST,
    # #               authentication_required=True,
    # #               signature_required=True)
    # # def submit_stop_overs(self, request, device_id):
    # #     if not isinstance(request.body, list):
    # #         raise XeberusService.InvalidArgumentException()
    # #
    # #     seed = request.get_argument('s',
    # #         data_type=HttpRequest.ArgumentDataType.string,
    # #         argument_passing=HttpRequest.ArgumentPassing.query_string,
    # #         is_required=False)
    # #
    # #     return XeberusService().submit_stop_overs(request.app_id, request.account_id, device_id,
    # #             [ Stopover.from_json(payload) for payload in request.body ])
    #
    #
    # @http_request(r'^/device/(device_id)$',
    #               http_method=HttpRequest.HttpMethod.PUT,
    #               authentication_required=True,
    #               signature_required=True)
    # def update_device(self, request, device_id):
    #     is_battery_alarm_muted = request.get_argument('is_battery_alarm_muted',
    #         data_type=HttpRequest.ArgumentDataType.boolean,
    #         is_required=False)
    #     is_device_alarm_muted = request.get_argument('is_device_alarm_muted',
    #         data_type=HttpRequest.ArgumentDataType.boolean,
    #         is_required=False)
    #     is_security_alarm_muted = request.get_argument('is_security_alarm_muted',
    #         data_type=HttpRequest.ArgumentDataType.boolean,
    #         is_required=False)
    #     security_level = request.get_argument('security_level',
    #         data_type=HttpRequest.ArgumentDataType.integer,
    #         is_required=False)
    #
    #     return XeberusService().update_device(request.app_id, request.account_id, device_id,
    #             is_battery_alarm_muted=is_battery_alarm_muted,
    #             is_device_alarm_muted=is_device_alarm_muted,
    #             is_security_alarm_muted=is_security_alarm_muted,
    #             security_level=security_level)



# Copyright (C) 2019 Majormode.  All rights reserved.
#
# This software is the confidential and proprietary information of
# Majormode or one of its subsidiaries.  You shall not disclose this
# confidential information and shall use it only in accordance with the
# terms of the license agreement or other applicable agreement you
# entered into with Majormode.
#
# MAJORMODE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
# OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
# TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE, OR NON-INFRINGEMENT.  MAJORMODE SHALL NOT BE LIABLE FOR ANY
# LOSSES OR DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
# OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.



# Copyright (C) 2019 Majormode.  All rights reserved.
#
# This software is the confidential and proprietary information of
# Majormode or one of its subsidiaries.  You shall not disclose this
# confidential information and shall use it only in accordance with the
# terms of the license agreement or other applicable agreement you
# entered into with Majormode.
#
# MAJORMODE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
# OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
# TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE, OR NON-INFRINGEMENT.  MAJORMODE SHALL NOT BE LIABLE FOR ANY
# LOSSES OR DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
# OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.

from majormode.perseus.constant.http import HttpMethod
from majormode.perseus.service.base_http_handler import HttpRequest
from majormode.perseus.service.base_http_handler import HttpRequestHandler
from majormode.perseus.service.base_http_handler import http_request
from majormode.xeberus.constant.sim import BillingFrequency
from majormode.xeberus.constant.sim import BillingMethod
from majormode.xeberus.constant.sim import Ownership
from majormode.xeberus.constant.sim import PaymentMethod

from majormode.xeberus.service.sim_card.sim_card_service import SimCardService


class SimCardServiceHttpRequestHandler(HttpRequestHandler):
    @http_request(r'^/sim_card$',
                  http_method=HttpMethod.GET,
                  authentication_required=True,
                  signature_required=True)
    def get_sim_cards(self, request):
        limit = request.get_argument(
            'limit',
            data_type=HttpRequest.ArgumentDataType.integer,
            is_required=False,
            default_value=SimCardService.DEFAULT_LIMIT)

        offset = request.get_argument(
            'offset',
            data_type=HttpRequest.ArgumentDataType.integer,
            is_required=False,
            default_value=0)

        sync_time = request.get_argument(
            'sync_time',
            data_type=HttpRequest.ArgumentDataType.timestamp,
            is_required=False)

        return SimCardService().get_sim_cards(
            request.app_id,
            request.account_id,
            limit=limit,
            offset=offset,
            sync_time=sync_time)

    @http_request(r'^/sim_card/(iccid)$',
                  http_method=HttpMethod.PUT,
                  authentication_required=True,
                  signature_required=True)
    def set_sim_card(self, request, iccid):
        balance = request.get_argument(
            'balance',
            data_type=HttpRequest.ArgumentDataType.decimal,
            is_required=False)

        billing_frequency = request.get_argument(
            'billing_frequency',
            data_type=HttpRequest.ArgumentDataType.enumeration,
            enumeration=BillingFrequency,
            is_required=False)

        billing_method = request.get_argument(
            'billing_method',
            data_type=HttpRequest.ArgumentDataType.enumeration,
            enumeration=BillingMethod,
            is_required=False)

        billing_time = request.get_argument(
            'billing_time',
            data_type = HttpRequest.ArgumentDataType.timestamp,
            is_required=False)

        ownership = request.get_argument(
            'ownership',
            data_type=HttpRequest.ArgumentDataType.enumeration,
            enumeration=Ownership,
            is_required=False)

        currency_code = request.get_argument(
            'currency_code',
            data_type=HttpRequest.ArgumentDataType.string,
            is_required=False)

        data_fee = request.get_argument(
            'data_fee',
            data_type=HttpRequest.ArgumentDataType.decimal,
            is_required=False)

        data_limit = request.get_argument(
            'data_limit',
            data_type=HttpRequest.ArgumentDataType.integer,
            is_required=False)

        data_used = request.get_argument(
            'data_used',
            data_type=HttpRequest.ArgumentDataType.integer,
            is_required=False)

        payment_method = request.get_argument(
            'payment_method',
            data_type=HttpRequest.ArgumentDataType.enumeration,
            enumeration=PaymentMethod,
            is_required=False)

        unlimited_data = request.get_argument(
            'unlimited_data',
            data_type = HttpRequest.ArgumentDataType.boolean,
            is_required = False)

        return SimCardService().set_sim_card(
            request.app_id,
            request.account_id,
            iccid,
            balance=balance,
            billing_frequency=billing_frequency,
            billing_method=billing_method,
            billing_time=billing_time,
            ownership=ownership,
            currency_code=currency_code,
            data_fee=data_fee,
            data_limit=data_limit,
            data_used=data_used,
            payment_method=payment_method,
            unlimited_data=unlimited_data)
